package solatium.gui.controller;

import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import lombok.extern.slf4j.Slf4j;
import solatium.blockchain.BlockChain;
import solatium.blockchain.Chain;
import solatium.interfaces.LoggedInUser;

import java.io.IOException;

@Slf4j
public class ControllerManager {

    private final Scene scene;
    private final Chain blockChain;
    private final LoggedInUser loggedInUser;

    public ControllerManager(Scene scene) {
        this.scene = scene;
        blockChain = new BlockChain();
        loggedInUser = LoggedInUser.INSTANCE;
    }

    public Chain getBlockChain() {
        return blockChain;
    }

    public LoggedInUser getLoggedInUser() {
        return loggedInUser;
    }

    public void showLoginScene() {
        log.info("set scene to login");
        String path = "/fxml/login.fxml";
        try {
            FXMLLoader loader = new FXMLLoader(getClass().getResource(path));
            scene.setRoot(loader.load());
            LoginController controller = loader.getController();
            controller.initManager(this);
        } catch (IOException e) {
            log.error("Failed to load {}", path);
            throw new FxmlLoadingException(String.format("Failed to load %s ", path));
        }
    }

    public void showProfileScene() {
        log.info("set scene to profile");
        String path = "/fxml/profile.fxml";
        try {
            FXMLLoader loader = new FXMLLoader(getClass().getResource(path));
            scene.setRoot(loader.load());
            ProfileController controller = loader.getController();
            controller.initManager(this);
        } catch (IOException e) {
            log.error("Failed to load {}", path);
            throw new FxmlLoadingException(String.format("Failed to load %s ", path));
        }
    }

    public void showPayInScene() {
        log.info("set scene to payin");
        String path = "/fxml/bankInteraction.fxml";
        try {
            FXMLLoader loader = new FXMLLoader(getClass().getResource(path));
            loader.setController(new PayinController());
            scene.setRoot(loader.load());
            PayinController controller = loader.getController();
            controller.initManager(this);
        } catch (IOException e) {
            log.error("Failed to load {}", path);
            throw new FxmlLoadingException(String.format("Failed to load %s ", path));
        }
    }

    public void showPayOutScene() {
        log.info("set scene to payout");
        String path = "/fxml/bankInteraction.fxml";
        try {
            FXMLLoader loader = new FXMLLoader(getClass().getResource(path));
            loader.setController(new PayoutController());
            scene.setRoot(loader.load());
            PayoutController controller = loader.getController();
            controller.initManager(this);
        } catch (IOException e) {
            log.error("Failed to load {}", path);
            throw new FxmlLoadingException(String.format("Failed to load %s ", path));
        }
    }
}
