package solatium.interfaces;

import org.junit.Test;

import static org.junit.Assert.*;

public class LoggedInUserTest {

    @Test
    public void shouldBeASingleton() {
        LoggedInUser loggedInUser = LoggedInUser.INSTANCE;
        LoggedInUser loggedInUser1 = LoggedInUser.INSTANCE;
        assertEquals(loggedInUser, loggedInUser1);
    }

    @Test
    public void shouldSetAndGetUserAsLoggedIn() throws SomeUserAlreadyLoggedInException, NoLoggedInUserException {
        LoggedInUser loggedInUser = LoggedInUser.INSTANCE;

        assertFalse(loggedInUser.isSomeoneLoggedIn());

        User user = new FakeUser();
        loggedInUser.setLoggedInUser(user);

        assertTrue(loggedInUser.isSomeoneLoggedIn());
        assertEquals(user.getUserName(), loggedInUser.getLoggedInUser().getUserName());
    }

    @Test (expected = NoLoggedInUserException.class)
    public void shouldThrowNoLoggedInUserException() throws NoLoggedInUserException {
        LoggedInUser loggedInUser = LoggedInUser.INSTANCE;

        try {
            loggedInUser.getLoggedInUser();
            fail();
        } catch (NoLoggedInUserException e) {
            assertEquals("No user is actually logged in", e.getMessage());
            throw e;
        }
    }

    @Test (expected = SomeUserAlreadyLoggedInException.class)
    public void shouldThrowSomeUserAlreadyLoggedInException() throws SomeUserAlreadyLoggedInException {
        LoggedInUser loggedInUser = LoggedInUser.INSTANCE;
        User user = new FakeUser();

        loggedInUser.setLoggedInUser(user);

        try {
            loggedInUser.setLoggedInUser(user);
            fail();
        } catch (SomeUserAlreadyLoggedInException e) {
            assertEquals("A user is already logged in!", e.getMessage());
            throw e;
        }
    }

    private final class FakeUser implements User {

        private final String userName = "userName";

        @Override
        public String getUserName() {
            return userName;
        }

        @Override
        public String getAddress() {
            return null;
        }

        @Override
        public String getPassword() {
            return null;
        }

        @Override
        public int getErrorCounter() {
            return 0;
        }

        @Override
        public int addErrorCounter() {
            return 0;
        }

        @Override
        public boolean isBlocked() {
            return false;
        }

        @Override
        public int getBalance() {
            return 0;
        }

        @Override
        public void send(String receiverAddress, int amount) {

        }
    }
}